package net.javajigi.user.dao;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;

import net.javajigi.user.model.RoleType;
import net.javajigi.user.model.User;

import org.springframework.dao.DataAccessException;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.jdbc.core.simple.ParameterizedRowMapper;
import org.springframework.jdbc.core.simple.SimpleJdbcDaoSupport;

public class SpringJDBCUserDao extends SimpleJdbcDaoSupport implements UserDao {
	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.dao.UserDAO#create(net.javajigi.user.model.User)
	 */
	public int insert(User user) throws DataAccessException {
		String query = "INSERT INTO USER VALUES (?, ?, ?, ?, ?)";
		return getSimpleJdbcTemplate().update(
				query,
				user.insert());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.dao.UserDAO#update(net.javajigi.user.model.User)
	 */
	public int update(User user) throws DataAccessException {
		String query = "UPDATE USER " +
                  	   "SET password=?, name=?, email=?, adminYN=? WHERE userId=?";
		return getSimpleJdbcTemplate().update(
				query,
				user.update());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.dao.UserDAO#remove(java.lang.String)
	 */
	public int delete(String userId) throws DataAccessException {
		String query = "DELETE FROM USER WHERE userId=?";
		return getSimpleJdbcTemplate().update(
				query,
				new Object[] { userId });
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.dao.UserDAO#findUser(java.lang.String)
	 */
	public User findUser(String userId) throws DataAccessException {
		String query = "SELECT " + 
        			   "userId, password, name, email, adminYN " + 
        			   "FROM USER WHERE userId=?";
		ParameterizedRowMapper<User> rowMapper = new ParameterizedRowMapper<User>() {
			public User mapRow(ResultSet rs, int rownum) throws SQLException {
				User user = new User();
				user.setUserId(rs.getString("userId"));
				user.setPassword(rs.getString("password"));
				user.setName(rs.getString("name"));
				user.setEmail(rs.getString("email"));
				user.setRoleType(RoleType.find(rs.getInt("adminYN")));

				return user;
			}
		};

		try {
			return getSimpleJdbcTemplate().queryForObject(query, rowMapper, new Object[] {userId});
		} catch (EmptyResultDataAccessException e) {
			return null;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.dao.UserDAO#findUserList()
	 */
	public List<User> findUserList() throws DataAccessException {
		String sql = "SELECT userId, name, email, adminYN FROM USER";
		ParameterizedRowMapper<User> rowMapper = new ParameterizedRowMapper<User>() {
			public User mapRow(ResultSet rs, int rownum) throws SQLException {
				User user = new User();
				user.setUserId(rs.getString("userId"));
				user.setName(rs.getString("name"));
				user.setEmail(rs.getString("email"));
				user.setRoleType(RoleType.find(rs.getInt("adminYN")));

				return user;
			}
		};

		return getSimpleJdbcTemplate().query(sql, rowMapper);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.javajigi.user.dao.UserDAO#existedUser(java.lang.String)
	 */
	public boolean existedUser(String userId) throws DataAccessException {
		String query = "SELECT count(*) FROM USER WHERE userId=? ";
		int result = getSimpleJdbcTemplate().queryForInt(
				query,
				new Object[] { userId });

		if (result == 1) {
			return true;
		} else {
			return false;
		}
	}

	public int countAdminUser() {
		String query = "SELECT count(userId) FROM USER WHERE adminYN=1";
		
		return getSimpleJdbcTemplate().queryForInt(query);
	}
}
