package net.javajigi.user.model;

import net.javajigi.user.ExceedAdminUserException;
import net.javajigi.user.ExistedUserException;
import net.javajigi.user.dao.UserDao;

import org.apache.commons.lang.builder.EqualsBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Configurable;

@Configurable
public class User {
	public static final int MAXIMUM_ADMIN_USER = 3;

	private String userId = null;
	private String password = null;
	private String password2 = null;
	private String name = null;
	private String email = null;
	private RoleType roleType;

	@Autowired
	private UserDao userDao;

	public User() {
	}

	public User(String userId, String password, String name, String email, RoleType roleType) {
		this.userId = userId;
		this.password = password;
		this.name = name;
		this.email = email;
		this.roleType = roleType;
	}

	public void setUserDao(UserDao userDao) {
		this.userDao = userDao;
	}

	public void setRoleType(RoleType roleType) {
		this.roleType = roleType;
	}
	
	public RoleType getRoleType() {
		return roleType;
	}

	public boolean isAdmin() {
		if( roleType == null ) {
			return false;
		}
		
		if( roleType == RoleType.ADMIN ) {
			return true;
		}

		return false;
	}

	public String getEmail() {
		return email;
	}

	public void setEmail(String email) {
		this.email = email;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getPassword2() {
		return password2;
	}

	public void setPassword2(String password2) {
		this.password2 = password2;
	}

	public String getUserId() {
		return userId;
	}

	public void setUserId(String userId) {
		this.userId = userId;
	}

	public boolean isMatchPassword(String inputPassword) {
		if (getPassword().equals(inputPassword)) {
			return true;
		} else {
			return false;
		}
	}

	boolean canCreate() {
		if (isAdmin() == false) {
			return true;
		}

		if (userDao.countAdminUser() >= MAXIMUM_ADMIN_USER) {
			return false;
		}
		
		return true;
	}

	public int create() throws ExceedAdminUserException, ExistedUserException {
		if (userDao.findUser(this.userId) != null) {
			throw new ExistedUserException(this.userId);
		}
		
		if( !canCreate() ) {
			throw new ExceedAdminUserException(this.userId);
		}

		return userDao.insert(this);
	}
	
	public Object[] insert() {
		return new Object[] { getUserId(), getPassword(), getName(), getEmail(), isAdmin() };
	}

	public Object[] update() {
		return new Object[] { getPassword(), getName(), getEmail(), isAdmin(), getUserId() };
	}
	
	@Override
	public boolean equals(Object obj) {
		if (obj instanceof User == false) {
			return false;
		}
		if (this == obj) {
			return true;
		}
		
		User user = (User)obj;
		return new EqualsBuilder()
					.append(getUserId(), user.getUserId())
					.append(getName(), user.getName())
					.append(getEmail(), user.getEmail())
					.append(getPassword(), user.getPassword())
					.isEquals();
	}
}
