package net.javajigi.user.service;


import net.javajigi.audit.AuditDao;
import net.javajigi.audit.AuditObject;
import net.javajigi.user.ExceedAdminUserException;
import net.javajigi.user.ExistedUserException;
import net.javajigi.user.model.AdminMockUser;
import net.javajigi.user.model.NormalMockUser;
import net.javajigi.user.model.User;

import org.jmock.Expectations;
import org.jmock.Mockery;
import org.junit.Before;
import org.junit.Test;

public class UserServiceTest {
	private Mockery context;
	private AuditDao auditDao;

	@Before	public void setUp() throws Exception {
		context = new Mockery();
		auditDao = context.mock(AuditDao.class);
	}
	
	@Test public void createUser() throws Exception {
		final User normalUser = new NormalUser();
		
		final AuditObject audit = new AuditObject(UserServiceImpl.NORMAL_USER + " : " + normalUser.getUserId(), UserServiceImpl.CREATE);
		context.checking(new Expectations() {{
			one(auditDao).log(audit);
		}});

		UserServiceImpl userService = new MockUserServiceImpl(auditDao);
		userService.addUser(normalUser);
		
		context.assertIsSatisfied();
	}
	
	@Test public void createAdmin() throws Exception {
		final User adminUser = new AdminUser();
		final AuditObject audit = new AuditObject(UserServiceImpl.ADMIN_USER  + " : " + adminUser.getUserId(), UserServiceImpl.CREATE);
		
		context.checking(new Expectations() {{
			one(auditDao).log(audit);
		}});
		
		UserServiceImpl userService = new MockUserServiceImpl(auditDao);
		userService.addUser(adminUser);
		
		context.assertIsSatisfied();
	}
	
	private class MockUserServiceImpl extends UserServiceImpl{
		private AuditDao auditDao;
		
		public MockUserServiceImpl(AuditDao auditDao) {
			this.auditDao = auditDao;
		}
		
		@Override
		public AuditObject getAudit() {
			AuditObject audit = new AuditObject();
			audit.setAuditDao(auditDao);
			
			return audit;
		}
	}
	
	private class NormalUser extends NormalMockUser {
		@Override
		public int create() throws ExceedAdminUserException,
				ExistedUserException {
			return 1;
		}
	}
	
	private class AdminUser extends AdminMockUser {
		@Override
		public int create() throws ExceedAdminUserException,
				ExistedUserException {
			return 1;
		}		
	}
}
